.hy 0
.TH pf n "March 1, 1991" "Osaka City Univ."
.ad
.SH NAME
pf \- Fortran callable plot package
.SH DESCRIPTION
"pfxxx" is a Fortran callable plot package. "plxxx" is a C callable package.
The following is the list of library subroutines with arguments. 
(i),(r),(s) indicate 'integer', 'real' and 'character string' 
variables, respectively. This uses the UGS fonts and their driver program
to draw texts.
.PP
.TP
.B pfclos \- Close the display device.
.nf

call pfclos
.fi
.TP
.B pflush \- Flush X draw
.nf

call pflush

.fi
.TP
.B pfgdev \- Get GDEVICE name
.nf

if( pfgdev( device ) ) then

	device:(s) Currently setenv GDEVICE name.
.fi
.TP
.B pfine \- Draw a line.
.nf

call pfine( x1, y1, x2, y2 )

	x1:(r) Starting X coordinate
	y1:(r) Starting Y coordinate
	x2:(r) End X coordinate
	y2:(r) End Y coordinate
.fi
.TP
.B pfopen \- Open the display device.
.nf

call pfopen( device )

	device:(s) Device name can be "necpc100", "pc98vt",
	           "kermit", "tek4010", "lbp8", "canonlbp",
	           "X", "XM", "XN", "XMN", "psl", "psp", "epsl",
		   "epsp", "gdevice". "lbp8" will print the 
		   picture on CANON laser printer in the 1st floor. 
		   "canonlbp" will print it on the 2nd floor. 
		   "psl" will print it on the LaserWriter in 
		   landscape, "psp" will print it on the 
		   LaserWriter in portrait.

.fi
.TP
.B pfoine \- Draw line with option
.nf

call pfoine( opt, x1, y1, x2, y2 )

	opt:(i) 0 for solid line, 1 for dashed line
	x1:(r) Starting X coordinate
	y1:(r) Starting Y coordinate
	x2:(r) End X coordinate
	y2:(r) End Y coordinate

.fi
.TP
.B pfovect \- Draw vector with option
.nf

call pfovect( opt, x, y, vis )

	opt:(i) 0 for solid line, 1 for dashed line
	x:(r) X coordinate
	y:(r) Y coordinate
	vis:(i) 0 for dark vector, 1 for visible vector

.fi
.TP
.B pfpr \- Print out saved plot.
"pfpr" prints the saved plot on the CANON laser printer in the 1st floor.
This has to be called after "pfclos".
.nf

call pfpr
.fi
.TP
.B pfsave \- Save the plot.
"pfsave" specifies plot-save option. The saved plot can be printer by "pfpr".
This has to be called before "pfopen".
.nf

call pfsave
.fi
.TP
.B pfscrn \- Specify the screen area.
.nf

call pfscrn( xl, yl, xh, yh )

	xl:(r) Lower X corner coordinate ( 0.0 - 1.0 )
	yl:(r) Lower Y corner coordinate ( 0.0 - 1.0 )
	xh:(r) Upper X corner coordinate ( 0.0 - 1.0 )
	yh:(r) Upper Y corner coordinate ( 0.0 - 1.0 )

	Full screen is pfscrn( 0.0, 0.0, 1.0, 1.0 ).
.fi
.TP
.B pftext \- Display texts.
.nf

call pftext( x, y, angle, size, font, text )

	( x, y ):(r) Coordinate of the 1st character of the text.
	angle:(r) Display angle in degrees.
	size:(r) Size of the text display.
	font:(i) 1 simplex, 2 duplex fonts, 3 UBC, 4 Greek.
	text:(s) Text string to display.
.fi
.TP
.B pfvect \- Draw a vector
.nf

call pfvect( x, y, vis )

	x:(r) X coordinate
	y:(r) Y coordinate
	vis:(i) 0 for dark vector, 1 for visible vector

.fi
.TP
.B pfxtext \- Display extended texts.
.nf

call pfxtext( x, y, angle, size, font, ptext, stext )

	( x, y ):(r) Coordinate of the 1st character of the text.
	angle:(r) Display angle in degrees.
	size:(r) Size of the text display.
	font:(i) 1 simplex, 2 duplex fonts, 3 UBC.
	ptext:(s) Primary text string to display.
	stext:(s) Secondary text string.
.fi
.TP
.B pfwdow \- Specify the display window 
.nf

call pfwdow( xl, yl, xh, yh )

	xl:(r) Lower X corner coordinate
	yl:(r) Lower Y corner coordinate
	xh:(r) Upper X corner coordinate
	yh:(r) Upper Y corner coordinate
.fi
.SH FILES
.nf
You have to link the following library,

Library:   /usrs/local/lib/libocu.a

.fi
.SH AUTHOR
This modification was done by Yoshiki Teramoto, Dept. of Physics, 
Osaka City University. Copyright of the UGS fonts belong to The Board 
of Trustees of the Leland, Stanford Junior University.
