      SUBROUTINE UGD001(FGLS,XCMU,YCMU)
C
C *******************  THE UNIFIED GRAPHICS SYSTEM  *******************
C *                      GENERATE LINE STRUCTURE                      *
C *                                                                   *
C *  THESE SUBROUTINES MAY BE USED TO GENERATE LINE STRUCTURE.        *
C *  SOLID LINES MAY BE BROKEN DOWN INTO:                             *
C *    1.  "DASHED" LINES,                                            *
C *    2.  "DOTTED" LINES, OR                                         *
C *    3.  "DOTDASH" LINES.                                           *
C *  SUBROUTINE UGD001 IS USED TO INITIALIZE THE PROCESS FOR A NEW    *
C *  SERIES OF LINE SEGMENTS, SUBROUTINE UGD002 IS USED TO SUPPLY     *
C *  AN END POINT TO THE LINE STRUCTURE GENERATING MODULES, AND       *
C *  SUBROUTINE UGD003 IS USED TO RETRIEVE A POINT OR LINE SEGMENT    *
C *  END POINT FROM THE LINE STRUCTURE GENERATING MODULES.  THE       *
C *  NORMAL SEQUENCE OF CALLS IS THE FOLLOWING: (1) UGD001 IS CALLED  *
C *  TO INITIALIZE PROCESSING, (2) UGD002 IS CALLED TO SUPPLY AN END  *
C *  POINT TO THE MODULES, (3) UGD003 IS CALLED REPEATEDLY TO         *
C *  RETRIEVE POINTS OR END POINTS OF LINES UNTIL IT SIGNALS THAT NO  *
C *  MORE DATA IS AVAILABLE, AND (4) STEP 2 IS REPEATED UNTIL NO      *
C *  MORE INPUT IS AVAILABLE.                                         *
C *                                                                   *
C *  THE CALLING SEQUENCES ARE:                                       *
C *    CALL UGD001(FGLS,XCMU,YCMU)                                    *
C *    CALL UGD002(BBIT,XCRD,YCRD)                                    *
C *    CALL UGD003(BBIT,XCRD,YCRD)                                    *
C *                                                                   *
C *  THE PARAMETERS IN THE CALLING SEQUENCES ARE:                     *
C *    FGLS  LINE STRUCTURE FLAG: 1 MEANS "DASHED"; 2 MEANS           *
C *          "DOTTED"; AND 3 MEANS "DOTDASH".                         *
C *    XCMU  CENTIMETERS PER UNIT DISTANCE IN THE X DIRECTION.        *
C *    YCMU  CENTIMETERS PER UNIT DISTANCE IN THE Y DIRECTION.        *
C *    BBIT  THE BLANKING BIT OR TERMINATION FLAG: 0 MEANS MOVE       *
C *          WITHOUT DRAWING AND 1 MEANS DRAW.  FOR UGD003, 2 MEANS   *
C *          A POINT IS AVAILABLE AND -1 MEANS NO MORE DATA IS        *
C *          AVAILABLE.                                               *
C *    XCRD  THE X COORDINATE OF A LINE END POINT OR A POINT.         *
C *    YCRD  THE Y COORDINATE OF A LINE END POINT OR A POINT.         *
C *                                                                   *
C *                          ROBERT C. BEACH                          *
C *                    COMPUTATION RESEARCH GROUP                     *
C *                STANFORD LINEAR ACCELERATOR CENTER                 *
C *                                                                   *
C *********************************************************************
C
      INTEGER       FGLS
      REAL          XCMU,YCMU
      INTEGER       BBIT
      REAL          XCRD,YCRD
C
      Include	'ugd000.for'
C
      INTEGER       INT1
      REAL          FLT1,FLT2,FLT3,FLT4,FLT5
C
C  SUBROUTINE UGD001: INITIALIZE THE LINE STRUCTURE GENERATION PROCESS.
      IFLS=FGLS
      CMUX=XCMU
      CMUY=YCMU
      FGAV=0
      GO TO (101,102,103), IFLS
  101 SZDS=0.333333333
      SZBK=0.333333333
      GO TO 401
  102 SZBK=0.25
      GO TO 401
  103 SZDS=0.25
      SZBK=0.25
      GO TO 401
C
C  SUBROUTINE UGD002: SUPPLY AN END POINT TO THE LINE STRUCTURE
C  GENERATION MODULES.
      ENTRY UGD002(BBIT,XCRD,YCRD)
      IF (BBIT.NE.0) GO TO 201
C  INITIALIZE FOR A NEW CONCATENATED LINE SEQUENCE.
      PNT2(1)=XCRD
      PNT2(2)=YCRD
      TVL2=0.0
      TVAL=0.0
      FGLO=0
      FGAV=1
      GO TO 401
C  PROCESS A CONTINUING LINE SEGMENT.
  201 FLT1=XCRD-PNT2(1)
      FLT2=YCRD-PNT2(2)
      FLT3=FLT1*CMUX
      FLT4=FLT2*CMUY
      FLT5=SQRT(FLT3*FLT3+FLT4*FLT4)
      IF (FLT5.LT.0.001) GO TO 401
      PNT1(1)=PNT2(1)
      PNT1(2)=PNT2(2)
      TVL1=TVL2
      PNT2(1)=XCRD
      PNT2(2)=YCRD
      TVL2=TVL1+FLT5
      DELX=FLT1/FLT5
      DELY=FLT2/FLT5
      FGAV=1
      GO TO 401
C
C  SUBROUTINE UGD003: RETRIEVE AN END POINT FROM THE LINE STRUCTURE
C  GENERATION MODULES.
      ENTRY UGD003(BBIT,XCRD,YCRD)
C  CHECK TO SEE IF ANY DATA IS AVAILABLE.
C    FGAV=0  MEANS NO DATA AVAILABLE.
C        =1  MEANS DATA AVAILABLE.
      IF (FGAV.NE.0) GO TO 301
      BBIT=-1
      GO TO 401
C
C  PROCESS DATA ACCORDING TO CURRENT LINE TYPE AND LAST OPERATION.
C    IFLS=1  MEANS DASHED LINE.
C        =2  MEANS DOTTED LINE.
C        =3  MEANS DOTDASH LINE.
C    FGLO=0  MEANS BLANK VECTOR SUPPLIED.
C        =1  MEANS DREW BLANK.
C        =2  MEANS DREW DASH.
C        =3  MEANS DREW POINT.
  301 INT1=3*FGLO+IFLS
      GO TO (302,309,311,
     X       304,402,312,
     X       307,402,313,
     X       402,310,314), INT1
C  DASHED LINE, BLANK VECTOR SUPPLIED.
  302 FGLO=1
      BBIT=0
  303 FGAV=0
      XCRD=PNT2(1)
      YCRD=PNT2(2)
      GO TO 401
C  DASHED LINE, DREW BLANK.
  304 IF ((TVAL+SZDS).GT.TVL2) GO TO 306
      TVAL=TVAL+SZDS
      FGLO=2
      BBIT=1
  305 XCRD=DELX*(TVAL-TVL1)+PNT1(1)
      YCRD=DELY*(TVAL-TVL1)+PNT1(2)
      GO TO 401
  306 BBIT=1
      GO TO 303
C  DASHED LINE, DREW DASH.
  307 IF ((TVAL+SZBK).GT.TVL2) GO TO 308
      TVAL=TVAL+SZBK
      FGLO=1
      BBIT=0
      GO TO 305
  308 FGAV=0
      BBIT=-1
      GO TO 401
C  DOTTED LINE, BLANK VECTOR SUPPLIED.
  309 FGLO=3
      BBIT=2
      GO TO 303
C  DOTTED LINE, DREW POINT.
  310 IF ((TVAL+SZBK).GT.TVL2) GO TO 308
      TVAL=TVAL+SZBK
      BBIT=2
      GO TO 305
C  DOTDASH LINE, BLANK VECTOR SUPPLIED.
  311 FGLO=1
      BBIT=0
      GO TO 303
C  DOTDASH LINE, DREW BLANK.
  312 IF ((TVAL+SZDS).GT.TVL2) GO TO 306
      TVAL=TVAL+SZDS
      FGLO=2
      BBIT=1
      GO TO 305
C  DOTDASH LINE, DREW DASH.
  313 IF ((TVAL+SZBK).GT.TVL2) GO TO 308
      TVAL=TVAL+SZBK
      FGLO=3
      BBIT=2
      GO TO 305
C  DOTDASH LINE, DREW POINT.
  314 IF ((TVAL+SZBK).GT.TVL2) GO TO 308
      TVAL=TVAL+SZBK
      FGLO=1
      BBIT=0
      GO TO 305
C
C  RETURN TO CALLING SUBROUTINE.
  401 RETURN
C  TERMINATE ON IMPOSSIBLE TRANSFER.
cccccccccc 27-Mar-95 Y.Teramoto modified the next line.
  402 CALL EXIT(0)
cccccccccc end of the modificaion.
      GO TO 401
C
      END
