      SUBROUTINE UGE001(FGDR,FGEQ,FGSC,FGMS,XCTR,YCTR,CSIZ,CANG,YFAC,
     X                  DATA,ERFG)
C
C *******************  THE UNIFIED GRAPHICS SYSTEM  *******************
C *   BREAK A PRIMARY/SECONDARY CHARACTER STRING DOWN INTO STROKES    *
C *                                                                   *
C *  THESE SUBROUTINES MAY BE USED TO BREAK A PRIMARY/SECONDARY PAIR  *
C *  OF CHARACTER STRINGS DOWN INTO INDIVIDUAL STROKES.  TWO          *
C *  DISTINCT OPERATIONS MAY BE PERFORMED:                            *
C *    1.  GENERATE THE ACTUAL STROKE END POINTS AND RETURN THEM.     *
C *    2.  SCAN THE CHARACTERS AND RETURN THE COORDINATES OF THE END  *
C *        OF THE STRING.  THE END OF STRING MAY BE:                  *
C *        A.  THE CENTER OF THE LAST CHARACTER.                      *
C *        B.  THE CENTER OF THE NEXT CHARACTER.                      *
C *  SUBROUTINE UGE001 IS USED TO INITIALIZE THE PROCESS FOR A NEW    *
C *  PAIR OF STRINGS.  SUBROUTINE UGE002 IS USED TO OBTAIN THE NEXT   *
C *  STROKE END POINT.  IF THE STROKES ARE ONLY BEING SCANNED,        *
C *  UGE002 NEED ONLY BE CALLED ONCE FOLLOWING THE CALL TO UGE001.    *
C *                                                                   *
C *  THE CALLING SEQUENCES ARE:                                       *
C *    CALL UGE001(FGDR,FGEQ,FGSC,FGMS,XCTR,YCTR,CSIZ,CANG,YFAC,      *
C *                DATA,ERFG)                                         *
C *    CALL UGE002(CHRP,CHRS,CHRN,DATA,BBIT,XCRD,YCRD,SZCH)           *
C *                                                                   *
C *  THE PARAMETERS IN THE CALLING SEQUENCES ARE:                     *
C *    FGDR  DRAW FLAG: 0 MEANS GENERATE STROKES; 1 MEANS SCAN/LAST;  *
C *          AND 2 MEANS SCAN/NEXT.                                   *
C *    FGEQ  EQUAL SPACING FLAG: 0 MEANS EQUAL SPACING; AND 1 MEANS   *
C *          PROPORTIONAL SPACING IF POSSIBLE.                        *
C *    FGSC  SPECIAL CHARACTER FLAG: 0 MEANS DO NOT PROCESS           *
C *          SUPERSCRIPT, SUBSCRIPT, ETC. CONTROL CHARACTERS; 1       *
C *          MEANS PROCESS THESE CHARACTERS.                          *
C *    FGMS  MISSING SECONDARY CHARACTERS FLAG: 0 MEANS CHRS WILL BE  *
C *          A DUMMY ARGUMENT; 1 MEANS IT WILL BE GIVEN.              *
C *    XCTR  THE X COORDINATE OF THE CENTER OF THE FIRST CHARACTER.   *
C *    YCTR  THE Y COORDINATE OF THE CENTER OF THE FIRST CHARACTER.   *
C *    CSIZ  THE SIZE OF THE CHARACTERS.                              *
C *    CANG  THE ANGLE THE CHARACTERS MAKE WITH THE HORIZONTAL.       *
C *    YFAC  THE Y FACTOR MULTIPLIER FOR THE OUTPUT.                  *
C *    DATA  THE BLOCK OF DATA DEFINING THE CHARACTER SET.            *
C *    ERFG  A FLAG WHICH WILL BE NON-ZERO IF DATA IS INVALID.        *
C *    CHRP  THE PRIMARY CHARACTER STRING.                            *
C *    CHRS  THE SECONDARY CHARACTER STRING.                          *
C *    CHRN  THE NUMBER OF CHARACTERS IN THE STRINGS.                 *
C *    BBIT  THE BLANKING BIT OR TERMINATION FLAG: -1 MEANS NO MORE   *
C *          DATA IS AVAILABLE; 0 MEANS MOVE WITHOUT DRAWING; AND 1   *
C *          MEANS DRAW.                                              *
C *    XCRD  THE X COORDINATE OF A STROKE END POINT OR THE END OF     *
C *          STRING X COORDINATE FOR A SCAN.                          *
C *    YCRD  THE Y COORDINATE OF A STROKE END POINT OR THE END OF     *
C *          STRING Y COORDINATE FOR A SCAN.                          *
C *    SZCH  THE SIZE CHANGE FACTOR.  THIS IS ONLY AVAILABLE WHEN     *
C *          BBIT IS -1.                                              *
C *                                                                   *
C *                          ROBERT C. BEACH                          *
C *                    COMPUTATION RESEARCH GROUP                     *
C *                STANFORD LINEAR ACCELERATOR CENTER                 *
C *                                                                   *
C *********************************************************************
C
      INTEGER       FGDR,FGEQ,FGSC,FGMS
      REAL          XCTR,YCTR,CSIZ,CANG,YFAC
      INTEGER*2     DATA(*)
      INTEGER       ERFG
      CHARACTER*(*) CHRP,CHRS
      INTEGER       CHRN,BBIT
      REAL          XCRD,YCRD,SZCH
C
      Include	'uge000.f'
C
      INTEGER*2     CHAR
      CHARACTER*2   CHRX
      EQUIVALENCE   (CHAR,CHRX)
C
      INTEGER       INT1,INT2,INT3
      REAL          FLT1,FLT2
C
C  SUBROUTINE UGE001: INITIALIZE THE STROKE GENERATION PROCESS.
      IFDR=FGDR
      IFEQ=FGEQ
      IFSC=FGSC
      IFMS=FGMS
      CHRO(1)=XCTR
      CHRO(2)=YCTR
      SIZE=CSIZ
      SNCS(1)=SIN(CANG/57.2957795)
      SNCS(2)=COS(CANG/57.2957795)
      XYFC=YFAC
      CSPC=REAL(DATA(7)+0)
      STEP=SIZE/CSPC
      XSZC=1.0
      SAV1(1)=CHRO(1)
      SAV1(2)=CHRO(2)
      SAV1(3)=STEP
      SAV1(4)=XSZC
      SAV2(1)=CHRO(1)
      SAV2(2)=CHRO(2)
      SAV2(3)=STEP
      SAV2(4)=XSZC
      SAV3(1)=CHRO(1)
      SAV3(2)=CHRO(2)
      SAV3(3)=STEP
      SAV3(4)=XSZC
      ICHR=1
      NCHR=DATA(6)
      PTCT=8
      PTOT=PTCT+NCHR
      PTLT=PTOT+NCHR
      ISTR=1
      NSTR=0
      IF (NCHR.LE.0) GO TO 604
      ERFG=0
      GO TO 603
C
C  SUBROUTINE UGE002: RETRIEVE AN INDIVIDUAL STROKE OR SCAN TO THE
C  END OF THE STRING.
      ENTRY UGE002(CHRP,CHRS,CHRN,DATA,BBIT,XCRD,YCRD,SZCH)
      IF (NCHR.LE.0) GO TO 601
C
C  CHECK TO SEE IF MORE STROKES ARE AVAILABLE.
  101 IF (ISTR.LE.NSTR) GO TO 102
      IF (NSTR.LE.0) GO TO 201
      IF (IFEQ.EQ.0) CALL UGE003(-0.5*XDSP*STEP,0.0)
      CALL UGE003(CSPC*STEP,0.0)
      GO TO 201
C  OBTAIN AND PROCESS THE NEXT STROKE.
  102 INT1=DATA(ISTR)
      ISTR=ISTR+1
      BBIT=INT1/16384
      INT1=INT1-16384*BBIT
      CALL UGE003(STEP*REAL((INT1/128)-64),
     X            STEP*REAL(MOD(INT1,128)-64))
      IF (IFDR.NE.0) GO TO 101
      IF (ISTR.LE.NSTR) GO TO 602
      GO TO 101
C
C  CHECK FOR END OF CHARACTER STRING.
  201 IF (ICHR.GT.CHRN) GO TO 601
      IF ((ICHR.EQ.CHRN).AND.(IFDR.EQ.1)) GO TO 601
C  SELECT NEXT CHARACTER.
      CHRX(1:1)=CHRP(ICHR:ICHR)
      IF (IFMS.EQ.0) THEN
        CHRX(2:2)=' '
      ELSE
        CHRX(2:2)=CHRS(ICHR:ICHR)
      END IF
      ICHR=ICHR+1
C
C  CHECK FOR SPECIAL CHARACTER PROCESSING.
      IF (IFSC.EQ.0) GO TO 501
      IF (CHRX(2:2).EQ.'X') GO TO 301
      IF (CHRX(2:2).EQ.'Y') GO TO 302
      IF (CHRX(2:2).EQ.'Z') GO TO 303
      GO TO 501
  301 IF (CHRX(1:1).EQ.'0') GO TO 401
      IF (CHRX(1:1).EQ.'1') GO TO 402
      IF (CHRX(1:1).EQ.'2') GO TO 403
      IF (CHRX(1:1).EQ.'3') GO TO 404
      GO TO 501
  302 IF (CHRX(1:1).EQ.'0') GO TO 405
      IF (CHRX(1:1).EQ.'1') GO TO 406
      GO TO 501
  303 IF (CHRX(1:1).EQ.'0') GO TO 407
      IF (CHRX(1:1).EQ.'1') GO TO 408
      IF (CHRX(1:1).EQ.'2') GO TO 409
      IF (CHRX(1:1).EQ.'3') GO TO 410
      IF (CHRX(1:1).EQ.'4') GO TO 411
      IF (CHRX(1:1).EQ.'5') GO TO 412
      IF (CHRX(1:1).EQ.'6') GO TO 413
      IF (CHRX(1:1).EQ.'7') GO TO 414
      GO TO 501
C
C  PROCESS SPECIAL CHARACTERS.
C  ENTER SUBSCRIPT MODE.
  401 FLT1=-0.111111111*CSPC*STEP
      FLT2=-0.500000000*CSPC*STEP
      STEP= 0.666666667*STEP
      XSZC= 0.666666667*XSZC
      GO TO 415
C  LEAVE SUBSCRIPT MODE.
  402 FLT1= 0.166666667*CSPC*STEP
      FLT2= 0.750000000*CSPC*STEP
      STEP= 1.500000000*STEP
      XSZC= 1.500000000*XSZC
      GO TO 415
C  ENTER SUPERSCRIPT MODE.
  403 FLT1=-0.111111111*CSPC*STEP
      FLT2= 0.500000000*CSPC*STEP
      STEP= 0.666666667*STEP
      XSZC= 0.666666667*XSZC
      GO TO 415
C  LEAVE SUPERSCRIPT MODE.
  404 FLT1= 0.166666667*CSPC*STEP
      FLT2=-0.750000000*CSPC*STEP
      STEP= 1.500000000*STEP
      XSZC= 1.500000000*XSZC
      GO TO 415
C  INCREASE SIZE BY ONE HALF.
  405 FLT1= 0.166666667*CSPC*STEP
      FLT2= 0.250000000*CSPC*STEP
      STEP= 1.500000000*STEP
      XSZC= 1.500000000*XSZC
      GO TO 415
C  DECREASE SIZE BY ONE THIRD.
  406 FLT1=-0.111111111*CSPC*STEP
      FLT2=-0.166666667*CSPC*STEP
      STEP= 0.666666667*STEP
      XSZC= 0.666666667*XSZC
      GO TO 415
C  PUT CURRENT STATE IN FIRST SAVE AREA.
  407 SAV1(1)=CHRO(1)
      SAV1(2)=CHRO(2)
      SAV1(3)=STEP
      SAV1(4)=XSZC
      GO TO 201
C  RESTORE STATE FROM FIRST SAVE AREA.
  408 CHRO(1)=SAV1(1)
      CHRO(2)=SAV1(2)
      STEP=SAV1(3)
      XSZC=SAV1(4)
      GO TO 201
C  PUT CURRENT STATE IN SECOND SAVE AREA.
  409 SAV2(1)=CHRO(1)
      SAV2(2)=CHRO(2)
      SAV2(3)=STEP
      SAV2(4)=XSZC
      GO TO 201
C  RESTORE STATE FROM SECOND SAVE AREA.
  410 CHRO(1)=SAV2(1)
      CHRO(2)=SAV2(2)
      STEP=SAV2(3)
      XSZC=SAV2(4)
      GO TO 201
C  PUT CURRENT STATE IN THIRD SAVE AREA.
  411 SAV3(1)=CHRO(1)
      SAV3(2)=CHRO(2)
      SAV3(3)=STEP
      SAV3(4)=XSZC
      GO TO 201
C  RESTORE STATE FROM THIRD SAVE AREA.
  412 CHRO(1)=SAV3(1)
      CHRO(2)=SAV3(2)
      STEP=SAV3(3)
      XSZC=SAV3(4)
      GO TO 201
C  PUT CURRENT STATE IN FOURTH SAVE AREA.
  413 SAV4(1)=CHRO(1)
      SAV4(2)=CHRO(2)
      SAV4(3)=STEP
      SAV4(4)=XSZC
      GO TO 201
C  RESTORE STATE FROM FOURTH SAVE AREA.
  414 CHRO(1)=SAV4(1)
      CHRO(2)=SAV4(2)
      STEP=SAV4(3)
      XSZC=SAV4(4)
      GO TO 201
C  FINISH SIZE-POSITION CHANGE OPERATIONS.
  415 CALL UGE003(FLT1,FLT2)
      GO TO 201
C
C  DO A BINARY SEARCH FOR THE CHARACTER IN THE DATA BLOCK.
  501 INT1=0
      INT2=NCHR-2
c*************** debug
cccc	print *, ' NCHR ', NCHR, ' PTCT ', PTCT, ' CHAR ', CHAR
cccc	print 973, ( IIII, DATA( PTCT + IIII ), IIII = 0, INT2 )
cccc973	format( 10( i3, 1x a1, ';' ) )
c********************
	do 504 iiii = 0, nchr - 2
	  if( data( ptct + iiii ) .eq. char ) then
	    int2 = iiii
	    go to 505
	  endif
504	continue
505	continue
cccc      IF (DATA(PTCT+INT1).NE.CHAR) THEN
cccc  502   IF (DATA(PTCT+INT2).NE.CHAR) THEN
cccc  503     IF (INT2.GT.(INT1+1)) THEN
cccc            INT3=(INT1+INT2)/2
cccc            IF (DATA(PTCT+INT3).GE.CHAR) THEN
cccc              INT2=INT3
cccc              GO TO 502
cccc            ELSE
cccc              INT1=INT3
cccc              GO TO 503
cccc            END IF
cccc          ELSE
cccc            INT2=NCHR-1
cccc          END IF
cccc        END IF
cccc      ELSE
cccc        INT2=INT1
cccc      END IF
c************* debug
cccc	print *, ' %%%%% end binary search: INT2 = ', INT2
cccc	print *, ' PTOT ', PTOT, ' PTLT ', PTLT
c********************
      INT1=DATA(PTOT+INT2)+PTLT-1
      INT2=DATA(INT1)
      XDSP=REAL(MOD(INT2,128)-64)
      ISTR=INT1+1
C******************* debug
cccc	print *, 'ISTR = ', ISTR
C***********************
      NSTR=(INT2/128)+ISTR-1
C
C  PROCESS THE CHARACTER.
      IF (IFEQ.EQ.0) CALL UGE003(-0.5*XDSP*STEP,0.0)
      GO TO 101
C
C  SIGNAL END OF PROCESSING AND RETURN TO CALLING SUBROUTINE.
  601 BBIT=-1
      SZCH=XSZC
  602 XCRD=CHRO(1)
      YCRD=CHRO(2)
  603 RETURN
C  TERMINATE ON UNAVAILABLE CHARACTER TABLE.
  604 ERFG=1
      GO TO 603
C
      END
      SUBROUTINE UGE003(XCRD,YCRD)
C
C *******************  THE UNIFIED GRAPHICS SYSTEM  *******************
C *     TRANSFORM COORDINATES FOR THE CHARACTER STROKE GENERATOR      *
C *                                                                   *
C *  THIS SUBROUTINE IS USED BY THE CHARACTER STROKE GENERATOR TO     *
C *  TRANSFORM POINTS.  THE RESULT OF THE TRANSFORMATION IS SAVED IN  *
C *  A COMMON BLOCK THAT IS SHARED BY THE STROKE GENERATOR.           *
C *                                                                   *
C *  THE CALLING SEQUENCE IS:                                         *
C *    CALL UGE003(XCRD,YCRD)                                         *
C *                                                                   *
C *  THE PARAMETERS IN THE CALLING SEQUENCE ARE:                      *
C *    XCRD  THE X COORDINATE OF THE POINT TO BE TRANSFORMED.         *
C *    YCRD  THE Y COORDINATE OF THE POINT TO BE TRANSFORMED.         *
C *                                                                   *
C *                          ROBERT C. BEACH                          *
C *                    COMPUTATION RESEARCH GROUP                     *
C *                STANFORD LINEAR ACCELERATOR CENTER                 *
C *                                                                   *
C *********************************************************************
C
      REAL          XCRD,YCRD
C
      Include	'uge000.f'
C
C  TRANSFORM THE POINT.
      CHRO(1)=     (SNCS(2)*XCRD-SNCS(1)*YCRD)+CHRO(1)
      CHRO(2)=XYFC*(SNCS(1)*XCRD+SNCS(2)*YCRD)+CHRO(2)
C
C  RETURN TO CALLING SUBROUTINE.
      RETURN
C
      END
